// Programmed by Seung S. Yang
// e-mail: syang1@kent.edu

import javax.swing.BoxLayout;
import javax.swing.border.TitledBorder;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.Timer;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.Hashtable;
import java.util.StringTokenizer;


class MedianetWeb implements ActionListener {
	public void actionPerformed(ActionEvent e) {
		String link = "http://www.medianet.kent.edu";
		String cmd = "cmd /c start /b "+ link;
		try {
			Runtime.getRuntime().exec(cmd);
		} catch (IOException eRun) {
			System.err.println(eRun);
		}
	}
}

class About implements ActionListener {
	static int aboutFrameWindowWidth = 500;
	static int aboutFrameWindowHeight = 200;
	public void actionPerformed(ActionEvent e) {
		JFrame aboutFrame = new JFrame("About");
		aboutFrame.getContentPane().add(new JLabel("<html><h2 align=center>Active Subnet Diffusion Transcoder<br>Channel Control Center Ver. 1.0</h2> <h3>This Control Center program is programmed by Seung S. Yang</h3></html>", JLabel.CENTER), BorderLayout.CENTER);
		aboutFrame.setSize(aboutFrameWindowWidth, aboutFrameWindowHeight);
		aboutFrame.setVisible(true);
	}
}

	
class StatusPanel extends JPanel {

	private JLabel statusLabels[] = new JLabel[5];

	public StatusPanel() {	
		setBorder(BorderFactory.createTitledBorder("Status"));
		statusLabels[0] = new JLabel(new ImageIcon("dixeeauth0.gif"));
		add(statusLabels[0]);
		
		statusLabels[1] = new JLabel(new ImageIcon("loadconfig0.gif"));
		add(statusLabels[1]);

		statusLabels[2] = new JLabel(new ImageIcon("resourceauth0.gif"));
		add(statusLabels[2]);

		statusLabels[3] = new JLabel(new ImageIcon("distribution0.gif"));
		add(statusLabels[3]);

		statusLabels[4] = new JLabel(new ImageIcon("ready0.gif"));
		add(statusLabels[4]);
	}

	protected void Update() {
		Thread thread = new Thread();
		try {
			thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		statusLabels[0].setIcon(new ImageIcon("dixeeauth1.gif"));
		validate();
		try {
			thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		statusLabels[0].setIcon(new ImageIcon("dixeeauth0.gif"));
		statusLabels[1].setIcon(new ImageIcon("loadconfig1.gif"));
		validate();
		try {
			thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		statusLabels[1].setIcon(new ImageIcon("loadconfig0.gif"));
		statusLabels[2].setIcon(new ImageIcon("resourceauth1.gif"));
		validate();
		try {
			thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		statusLabels[2].setIcon(new ImageIcon("resourceauth0.gif"));
		statusLabels[3].setIcon(new ImageIcon("distribution1.gif"));
		validate();
		try {
			thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		statusLabels[3].setIcon(new ImageIcon("distribution0.gif"));
		statusLabels[4].setIcon(new ImageIcon("ready1.gif"));
		validate();
	}
}

public class ControlCenter extends JPanel {
	static int windowHeight = 640;
	static int windowWidth = 500;
	static int bottomWindowHeight = 140;
	static int PFM_INIT = 1;
	static int FPS_INIT = 0;
	static int FRAME_SLIDER_PORT = 7001;
	static int DELAY_FACTOR = 300;
	static int CMD_PORT = 9000;
	static int LOAD_CONTROL_PORT = 9900;
	static int MAX_ENCODER_NO = 30;

	private JList encList;
	private JSlider frameSlider = new JSlider(0, 40, FPS_INIT);
	
	private JTextField svrName = new JTextField();
	private JTextField svrPort = new JTextField();
	private JTextField decName = new JTextField();
	private JTextField decPort = new JTextField();
	private JTextField muxName = new JTextField();
	private JTextField muxPort = new JTextField();
	private JTextField playerName = new JTextField();
	private JTextField playerPort = new JTextField();
	private JTextField encCmdPort = new JTextField();
	private JTextField encDataPort = new JTextField();

	private ImageIcon dukeImageIcons[] = new ImageIcon[10];
	private JLabel picture;
	private Timer timer;
	private int delay = DELAY_FACTOR;
	private int iconNo = 0;

	private DefaultListModel encListModel;

	private JPanel diagramPanel;
	private JPanel loadControlPanel;
	private JSlider loadControlSlider[] = new JSlider[MAX_ENCODER_NO];
	
	public ControlCenter() {
	
		JTabbedPane tabbedPane = new JTabbedPane();
		
		Component runPanel = MakeRunPanel();
		tabbedPane.addTab("Run", null, runPanel,"Run");
		tabbedPane.setSelectedIndex(0);

		Component cfgPanel = MakeConfigPanel();
		tabbedPane.addTab("Configure", null, cfgPanel,"Configure");

		Component bitRateControlPanel = MakeBitRateControlPanel();
		tabbedPane.addTab("Bit rate control", null, bitRateControlPanel, "Bit rate control");

		setLayout(new GridLayout(2,1));
		add(tabbedPane);

		diagramPanel = (JPanel)MakeDiagramPanel();
		add(diagramPanel);


		loadControlPanel = (JPanel)MakeLoadControlPanel();
		tabbedPane.addTab("Stress emulator", null, loadControlPanel, "Stress emulator");
		UpdateLoadControlPanel();

	}
	
	protected Component MakeRunPanel() {
		
		// Top Panel
		JPanel panel = new JPanel(false);
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();
		rightPanel.setPreferredSize(new Dimension((int)(windowWidth*0.7), windowHeight));
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createHorizontalGlue());
		panel.add(leftPanel);
		panel.add(Box.createHorizontalGlue());
		panel.add(rightPanel);
		panel.add(Box.createHorizontalGlue());
		
		// Left Panel
		leftPanel.setLayout(new GridLayout(5,1,0,5));
		JButton startButton = new JButton("Start");
		JButton stopButton = new JButton("Stop");
		JButton resetButton = new JButton("Reset");
		startButton.addActionListener(new Start());
		stopButton.addActionListener(new Stop());
		resetButton.addActionListener(new Reset());
		leftPanel.add(new Box(BoxLayout.X_AXIS));
		leftPanel.add(startButton);
		leftPanel.add(stopButton);
		leftPanel.add(resetButton);

		// Right Panel
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
		// Performance Panel
		JPanel perfPanel = new JPanel();
		perfPanel.setBorder(BorderFactory.createTitledBorder("Performance meter (%)"));
		perfPanel.setLayout(new GridLayout(1,1));
		JSlider perfSlider = new JSlider(1, 100, PFM_INIT);
		Hashtable perfLabelTable = new Hashtable();
		perfLabelTable.put(new Integer(1), new JLabel("Low"));
		perfLabelTable.put(new Integer(50), new JLabel("Medium"));
		perfLabelTable.put(new Integer(100), new JLabel("High"));
		perfSlider.setLabelTable(perfLabelTable);
		perfSlider.setPaintLabels(true);
		perfSlider.setMajorTickSpacing(10);
		perfSlider.setPaintTicks(true);
		perfSlider.addChangeListener(new PerfChange());
		perfPanel.add(perfSlider);

		// Frame rate Panel
		JPanel framePanel = new JPanel();
		framePanel.setLayout(new BoxLayout(framePanel, BoxLayout.X_AXIS));
		JPanel frameSliderPanel = new JPanel();
		TitledBorder frameSliderTitledBorder = BorderFactory.createTitledBorder("Current frame rates (fps)");
		frameSliderTitledBorder.setTitleColor(Color.gray);
		frameSliderPanel.setBorder(frameSliderTitledBorder);
		frameSliderPanel.setLayout(new GridLayout(1,1));
		frameSlider.setMajorTickSpacing(10);
		frameSlider.setMinorTickSpacing(1);
		frameSlider.setPaintTicks(true);
		frameSlider.setPaintLabels(true);
		frameSliderPanel.add(frameSlider);
		framePanel.add(frameSliderPanel);

		FrameSliderUpdater frameSliderHandler = new FrameSliderUpdater();
		frameSliderHandler.start();

		// Animation Panel
		for (int i = 1; i <= 10; i++) {
			dukeImageIcons[i-1] = new ImageIcon("T"+i+".gif");
		}
		picture = new JLabel(dukeImageIcons[0], JLabel.CENTER);
		picture.setAlignmentX(Component.CENTER_ALIGNMENT);
		framePanel.add(picture);
		timer = new Timer(delay, new AnimationAction());
		
		startAnimation();

		rightPanel.add(perfPanel);
		rightPanel.add(framePanel);

		return panel;
	}
	
	public void startAnimation() {
		timer.start();
	}
	
	public void stopAnimation() {
		timer.stop();
	}
	
	protected Component MakeConfigPanel() {
		JPanel panel = new JPanel(false);
		JPanel leftPanel = new JPanel(false);
		JPanel rightPanel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

		// Left Panel		
		leftPanel.setLayout(new GridLayout(6,3));
		leftPanel.add(Box.createHorizontalGlue());
		leftPanel.add(new JLabel("Host Name"));
		leftPanel.add(new JLabel("Port Number"));
		
		leftPanel.add(new JLabel("Video Server"));
		leftPanel.add(svrName);
		leftPanel.add(svrPort);
		
		leftPanel.add(new JLabel("Decoder"));
		leftPanel.add(decName);
		leftPanel.add(decPort);

		leftPanel.add(new JLabel("Multiplexor"));
		leftPanel.add(muxName);
		leftPanel.add(muxPort);

		leftPanel.add(new JLabel("Player"));
		leftPanel.add(playerName);
		leftPanel.add(playerPort);
		
		JButton DistButton = new JButton("Distribute");
		DistButton.addActionListener(new Distribute());
		leftPanel.add(DistButton);

		// Right Panel
		rightPanel.setPreferredSize(new Dimension((int)(windowWidth*0.4), windowHeight));
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		rightPanel.setLayout(gridbag);
		//c.anchor = GridBagConstraints.NORTH;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		
		JLabel encLabel = new JLabel("Encoders", JLabel.CENTER);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(encLabel, c);
		rightPanel.add(encLabel);
		
		JPanel encPanel = new JPanel();
		encPanel.setLayout(new BoxLayout(encPanel, BoxLayout.X_AXIS));
		JTextField encName = new JTextField();
		encName.addActionListener(new EncEnter());
		gridbag.setConstraints(encName, c);
		encPanel.add(encName);
		JButton encCancelButton = new JButton("Remove");
		encCancelButton.addActionListener(new EncCancel());
		encPanel.add(encCancelButton);
		//c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.gridy = 1;
		gridbag.setConstraints(encPanel, c);
		rightPanel.add(encPanel);
		
		encListModel = new DefaultListModel();
		encList = new JList(encListModel);
		encList.setVisibleRowCount(3);
		encList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane encListScrollPane = new JScrollPane(encList);
		c.gridx = 0;
		c.gridy = 2;
		c.gridheight = 4;
		c.ipady = 25;
		gridbag.setConstraints(encListScrollPane, c);
		rightPanel.add(encListScrollPane);
		
		JPanel encPortPanel = new JPanel();
		encPortPanel.setLayout(new GridLayout(2,2));
		encPortPanel.add(new JLabel("CmdPort "));
		encPortPanel.add(new JLabel(" DataPort"));
		encPortPanel.add(encCmdPort);
		encPortPanel.add(encDataPort);

		rightPanel.add(encPortPanel);
		
		panel.add(leftPanel);
		panel.add(rightPanel);
		
		LoadConfigFile();
		panel.updateUI();
		return panel;
	}

	protected Component MakeBitRateControlPanel() {
		JPanel panel = new JPanel(false);
		
		JPanel bitRatePanel = new JPanel();
		bitRatePanel.setBorder(BorderFactory.createTitledBorder("Bit rate"));
		bitRatePanel.setLayout(new GridLayout(1,1));
		
		JSlider bitRateSlider = new JSlider(0, 600, PFM_INIT);
		Hashtable bitRateLabelTable = new Hashtable();
		bitRateLabelTable.put(new Integer(0), new JLabel("Low (0: Disable)"));
		bitRateLabelTable.put(new Integer(300), new JLabel("Medium"));
		bitRateLabelTable.put(new Integer(600), new JLabel("High"));
		bitRateSlider.setLabelTable(bitRateLabelTable);
		bitRateSlider.setPaintLabels(true);
		bitRateSlider.setMajorTickSpacing(100);
		bitRateSlider.setPaintTicks(true);
		bitRateSlider.addChangeListener(new BitRateChange());

		bitRatePanel.add(bitRateSlider);

		bitRatePanel.setPreferredSize(new Dimension(400, 70));
		panel.add(bitRatePanel);
		return panel;
	}

	protected Component MakeLoadControlPanel() {
		JPanel panel = new JPanel(false);

		return panel;
	}
	
	protected void UpdateLoadControlPanel() {
		// Update Load Control Slider
		loadControlPanel.removeAll();
		JPanel subPanel[] = new JPanel[encListModel.getSize()];

		for (int i=0; i < MAX_ENCODER_NO; i++) {
			loadControlSlider[i] = null;
		}

		Hashtable loadControlLabelTable = new Hashtable();
		loadControlLabelTable.put(new Integer(1), new JLabel("Low"));
		loadControlLabelTable.put(new Integer(500), new JLabel("Medium"));
		loadControlLabelTable.put(new Integer(1000), new JLabel("High"));

		for (int i=0; i < encListModel.getSize(); i++) {
			subPanel[i] = new JPanel();
			subPanel[i].setBorder(BorderFactory.createTitledBorder(encListModel.getElementAt(i).toString()));
			subPanel[i].setLayout(new GridLayout(1,1));
			loadControlSlider[i] = new JSlider(1, 1000, PFM_INIT);
			loadControlSlider[i].setLabelTable(loadControlLabelTable);
			loadControlSlider[i].setPaintLabels(true);
			loadControlSlider[i].setMajorTickSpacing(100);
			loadControlSlider[i].setPaintTicks(true);
			loadControlSlider[i].addChangeListener(new LoadControlChange());
			subPanel[i].add(loadControlSlider[i]);
			loadControlPanel.add(subPanel[i]);
		}
		loadControlPanel.validate();
	}

	protected Component MakeDiagramPanel() {
		
		JPanel panel = new JPanel(false);

		diagramPanel = panel;
		// Diagram Panel
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createHorizontalGlue());
		panel.add(GetServerDiagram());
		//panel.add(Box.createHorizontalGlue());
		panel.add(GetDecoderDiagram());
		//panel.add(Box.createHorizontalGlue());
		panel.add(GetEncoderDiagram());
		//panel.add(Box.createHorizontalGlue());
		panel.add(GetMuxDiagram());
		//panel.add(Box.createHorizontalGlue());
		panel.add(GetPlayerDiagram());
		panel.add(Box.createHorizontalGlue());

		return panel;
	}

	protected void UpdateDiagramPanel() {
		diagramPanel.removeAll();
		diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.add(GetServerDiagram());
		//diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.add(GetDecoderDiagram());
		//diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.add(GetEncoderDiagram());
		//diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.add(GetMuxDiagram());
		//diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.add(GetPlayerDiagram());
		diagramPanel.add(Box.createHorizontalGlue());
		diagramPanel.update(diagramPanel.getGraphics());
		diagramPanel.validate();
	}
	
	

	protected Component GetServerDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String svrNameString = new String(svrName.getText());
		if (svrNameString == "")
			return panel;


		panel.setBorder(BorderFactory.createTitledBorder(svrNameString));
		panel.add(new JLabel(new ImageIcon("server.gif"), JLabel.CENTER));

		if (svrNameString.compareTo(decName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("decoder.gif"), JLabel.CENTER));
		}
		
		if (svrNameString.compareTo(muxName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("mux.gif"), JLabel.CENTER));
		}
		
		if (svrNameString.compareTo(playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("player.gif"), JLabel.CENTER));
		}

		for (int i = 0; i < encListModel.getSize(); i++) {
			if (svrNameString.compareTo((encListModel.getElementAt(i)).toString()) == 0) {
				panel.add(new JLabel(new ImageIcon("encoder.gif"), JLabel.CENTER));
			}
		}
		
		return panel;
	}
	
	protected Component GetDecoderDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String decNameString = new String(decName.getText());
		if (decNameString == "")
			return panel;

		if (decNameString.compareTo(svrName.getText()) == 0)
			return panel;

		diagramPanel.add(new JLabel(new ImageIcon("connection.gif"), JLabel.CENTER));
		panel.setBorder(BorderFactory.createTitledBorder(decNameString));
		panel.add(new JLabel(new ImageIcon("decoder.gif"), JLabel.CENTER));

		if (decNameString.compareTo(muxName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("mux.gif"), JLabel.CENTER));
		}
		
		if (decNameString.compareTo(playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("player.gif"), JLabel.CENTER));
		}

		for (int i = 0; i < encListModel.getSize(); i++) {
			if (decNameString.compareTo((encListModel.getElementAt(i)).toString()) == 0) {
				panel.add(new JLabel(new ImageIcon("encoder.gif"), JLabel.CENTER));
			}
		}

		return panel;
	}

	protected Component GetEncoderDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		diagramPanel.add(new JLabel(new ImageIcon("connection.gif"), JLabel.CENTER));

		for (int i=0; i < encListModel.getSize(); i++) {
			JPanel subPanel = new JPanel(false);
			subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
			String encNameString = new String(encListModel.getElementAt(i).toString());
			if (encNameString == "") {
				panel.add(subPanel);
				continue;
			}

			if (encNameString.compareTo(svrName.getText()) == 0) {
				panel.add(subPanel);
				continue;
			}

			if (encNameString.compareTo(decName.getText()) == 0) {
				panel.add(subPanel);
				continue;
			}

			subPanel.setBorder(BorderFactory.createTitledBorder(encNameString));
			subPanel.add(new JLabel(new ImageIcon("encoder.gif"), JLabel.CENTER));

			if (encNameString.compareTo(muxName.getText()) == 0) {
				subPanel.add(new JLabel(new ImageIcon("mux.gif"), JLabel.CENTER));
			}
		
			if (encNameString.compareTo(playerName.getText()) == 0) {
				subPanel.add(new JLabel(new ImageIcon("player.gif"), JLabel.CENTER));
			}
			panel.add(subPanel);
		}
		return panel;
	}

	protected Component GetMuxDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String muxNameString = new String(muxName.getText());
		if (muxNameString == "")
			return panel;

		if (muxNameString.compareTo(svrName.getText()) == 0)
			return panel;
			
		if (muxNameString.compareTo(decName.getText()) == 0)
			return panel;

		for (int i = 0; i < encListModel.getSize(); i++) {
			if (muxNameString.compareTo((encListModel.getElementAt(i)).toString()) == 0) {
				return panel;
			}
		}

		diagramPanel.add(new JLabel(new ImageIcon("connection.gif"), JLabel.CENTER));		
		panel.setBorder(BorderFactory.createTitledBorder(muxNameString));
		panel.add(new JLabel(new ImageIcon("mux.gif"), JLabel.CENTER));

		if (muxNameString.compareTo(playerName.getText()) == 0) {
			panel.add(new JLabel(new ImageIcon("player.gif"), JLabel.CENTER));
		}

		return panel;
	}

	protected Component GetPlayerDiagram() {
		JPanel panel = new JPanel(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		String playerNameString = new String(playerName.getText());
		if (playerNameString == "")
			return panel;

		if (playerNameString.compareTo(svrName.getText()) == 0)
			return panel;
			
		if (playerNameString.compareTo(decName.getText()) == 0)
			return panel;

		for (int i = 0; i < encListModel.getSize(); i++) {
			if (playerNameString.compareTo((encListModel.getElementAt(i)).toString()) == 0) {
				return panel;
			}
		}

		diagramPanel.add(new JLabel(new ImageIcon("connection.gif"), JLabel.CENTER));		
		panel.setBorder(BorderFactory.createTitledBorder(playerNameString));
		panel.add(new JLabel(new ImageIcon("player.gif"), JLabel.CENTER));

		return panel;
	}

	protected void LoadConfigFile() {
		try {
			BufferedReader confFile = new BufferedReader(new InputStreamReader(new FileInputStream("PXNet.conf")));
			String s;
			StringTokenizer t;

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			svrName.setText(t.nextToken());
			svrPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			playerName.setText(t.nextToken());
			playerPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			s = t.nextToken();
			encCmdPort.setText(t.nextToken());
			encDataPort.setText(t.nextToken());

			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");		
			decName.setText(t.nextToken());
			decPort.setText(t.nextToken());
			
			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			muxName.setText(t.nextToken());
			muxPort.setText(t.nextToken());
			
			s = confFile.readLine();
			t = new StringTokenizer(s, " \t\n\r");
			s = t.nextToken();
			int encNumber;
			encNumber = (new Integer(t.nextToken())).intValue();
			int i;
			for (i = 0; i < encNumber; i++) {
				s = confFile.readLine();
				t = new StringTokenizer(s, " \t\n\r");
				encListModel.addElement(t.nextToken());
			}
			confFile.close();
		} catch (IOException e) {
			System.err.println(e);
		}
	}

	protected void SendMessage(String hostName, String message)
	{
		try {
			Socket hostCtrlSocket = new Socket(hostName, CMD_PORT);
			PrintStream hostCtrlOutputStream = new PrintStream(hostCtrlSocket.getOutputStream());
			hostCtrlOutputStream.print(message);
			hostCtrlOutputStream.flush();
			hostCtrlSocket.close();
		} catch (UnknownHostException sE) {
			System.err.println(sE);
		} catch (IOException sE) {
			System.err.println(sE);
		}
	}

	protected void SendConfigFile(String hostName)
	{
		// Send configuration to host
		try {
			Socket ctrlSocket = new Socket(hostName, CMD_PORT);
			PrintStream ctrlOutputStream = new PrintStream(ctrlSocket.getOutputStream());
			ctrlOutputStream.print("download PXNet.conf");
			ctrlOutputStream.flush();
			ctrlSocket.close();

			Socket dataSocket = new Socket(hostName, CMD_PORT);
			BufferedOutputStream dataOutputStream = new BufferedOutputStream(dataSocket.getOutputStream());
			BufferedInputStream dataInputStream = new BufferedInputStream( new FileInputStream("PXNet.conf"));
			int c;
			while ((c = dataInputStream.read()) != -1) {
				dataOutputStream.write(c);
			}
			dataOutputStream.flush();
			dataInputStream.close();
			dataSocket.close();
		} catch (UnknownHostException sE) {
			System.err.println(sE);
		} catch (IOException sE) {
			System.err.println(sE);
		}
	}

	class AnimationAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int sliderValue = frameSlider.getValue();
			if (sliderValue != 0) {
				iconNo = ++iconNo % 10;
				picture.setIcon(dukeImageIcons[iconNo]);
				delay = DELAY_FACTOR/sliderValue;
				timer.setDelay(delay);
				timer.setInitialDelay(delay * 10);
				timer.restart();
			}
		}
	}

	class Start implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			// Server start
			SendMessage(svrName.getText(), "start svr");
			// Encoders start
			int i;
			for (i = 0; i < encListModel.getSize(); i++) {
				SendMessage((encListModel.getElementAt(i)).toString(), "start enc");
			}
			// Mux start
			SendMessage(muxName.getText(), "start mux");
			// Decoder start
			SendMessage(decName.getText(), "start dec");
			// Player
			String cmd = "cmd /c start /b "+ "XPlayer.exe";
/* For running player seperate machine
			try {
				Runtime.getRuntime().exec(cmd);
			} catch (IOException eRun) {
				System.err.println(eRun);
			}
*/
			Toolkit.getDefaultToolkit().beep();
		}
	}
	
	class Stop implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			// Server stop
			SendMessage(svrName.getText(), "stop svr");
			// Encoders stop
			int i;
			for (i = 0; i < encListModel.getSize(); i++) {
				SendMessage((encListModel.getElementAt(i)).toString(), "stop enc");
			}
			// Mux stop
			SendMessage(muxName.getText(), "stop mux");
			// Decoder stop
			SendMessage(decName.getText(), "stop dec");
			Toolkit.getDefaultToolkit().beep();
		}
	}

	class Reset implements ActionListener {
		public void actionPerformed(ActionEvent e) {
/* for debugging
			if (muxName.getText() != "") {
				try {
					Socket xmuxCtrlSocket = new Socket(muxName.getText(), (new Integer(muxPort.getText())).intValue() + 1);
					PrintStream xmuxOutputStream = new PrintStream(xmuxCtrlSocket.getOutputStream());
					xmuxOutputStream.print((int)((JSlider)e.getSource()).getValue());
					xmuxOutputStream.flush();
					xmuxCtrlSocket.close();
				} catch (UnknownHostException sE) {
					System.err.println(sE);
				} catch (IOException sE) {
					System.err.println(sE);
				}
			}
*/
			// Server start
			SendMessage(svrName.getText(), "start svr");
			// Encoders start
			int i;
			for (i = 0; i < encListModel.getSize(); i++) {
				SendMessage((encListModel.getElementAt(i)).toString(), "start enc");
			}
			// Mux start
			SendMessage(muxName.getText(), "start mux");
			// Decoder start
			SendMessage(decName.getText(), "start dec");

			// Server stop
			SendMessage(svrName.getText(), "stop svr");
			// Encoders stop
			for (i = 0; i < encListModel.getSize(); i++) {
				SendMessage((encListModel.getElementAt(i)).toString(), "stop enc");
			}
			// Mux stop
			SendMessage(muxName.getText(), "stop mux");
			// Decoder stop
			SendMessage(decName.getText(), "stop dec");

			Toolkit.getDefaultToolkit().beep();
		}
	}
	
	class PerfChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			if (!source.getValueIsAdjusting()) {
				if (decName.getText() != "") {
					int perf = (int)((JSlider)e.getSource()).getValue();
					try {
						Socket decSocket = new Socket(decName.getText(), (new Integer(decPort.getText())).intValue());
						PrintStream decOutputStream = new PrintStream(decSocket.getOutputStream());
						decOutputStream.print(perf);
						decOutputStream.flush();
						decSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
					System.out.println("Perf Value = " + perf);
				}
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

	class BitRateChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			if (!source.getValueIsAdjusting()) {
				int bitRate = (int)((JSlider)e.getSource()).getValue();
				for (int i = 0; i < encListModel.getSize(); i++) {
					try {
						Socket ctrlSocket = new Socket((encListModel.getElementAt(i)).toString(), CMD_PORT);
						PrintStream ctrlOutputStream = new PrintStream(ctrlSocket.getOutputStream());
						ctrlOutputStream.print("bitrate " + bitRate);
						ctrlOutputStream.flush();
						ctrlSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
				}
				System.out.println("Bit rate Value = " + bitRate);
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

	class Distribute implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int i;
/* for debugging
			System.out.println("Server " + svrName.getText() + ":" + svrPort.getText());
			System.out.println("Decoder " + decName.getText() + ":" + decPort.getText());
			System.out.println("Multiplexer " + muxName.getText() + ":" + muxPort.getText());
			System.out.println("Player " + playerName.getText() + ":" + playerPort.getText());
			System.out.println("Encoders ("+ encListModel.getSize()+")");
			for (i = 0; i < encListModel.getSize(); i++) {
				System.out.println((encListModel.getElementAt(i)).toString());
			}
			System.out.println("Encoder Ports " + encCmdPort.getText() + " : " + encDataPort.getText());
*/
			// Write configuration file to local disk
			try {
				PrintWriter confFile = new PrintWriter(new BufferedWriter(new FileWriter("PXNet.conf")));
				confFile.println(svrName.getText() + "\t\t" + svrPort.getText());
				confFile.println(playerName.getText() + "\t\t" + playerPort.getText());
				confFile.println("EncLuncher\t" + encCmdPort.getText() + "\t" + encDataPort.getText());
				confFile.println(decName.getText() + "\t\t" + decPort.getText());
				confFile.println(muxName.getText() + "\t\t" + muxPort.getText());
				confFile.println("EncLuncherNo" + "\t" + encListModel.getSize());
				for (i = 0; i < encListModel.getSize(); i++) {
					confFile.println((encListModel.getElementAt(i)).toString());
				}
				confFile.close();
			} catch (IOException fE) {
				System.err.println(fE);
			}
			UpdateDiagramPanel();
			UpdateLoadControlPanel();

			// Send configuration to server
			SendConfigFile(svrName.getText());
			// Send configuration to decoder
			SendConfigFile(decName.getText());
			// Send configuration to mux
			SendConfigFile(muxName.getText());
			// Send configuration to enc
			for (i = 0; i < encListModel.getSize(); i++) {
				SendConfigFile((encListModel.getElementAt(i)).toString());
			}

			Toolkit.getDefaultToolkit().beep();
		}
	}

	class EncEnter implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JTextField txtField = (JTextField)e.getSource();
			String text = txtField.getText();
			txtField.setText("");
			Toolkit.getDefaultToolkit().beep();
			encListModel.addElement(text);
		}
	}
	
	class EncCancel implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			int index = encList.getSelectedIndex();
			encListModel.remove(index);
		}
	}

	class FrameSliderUpdater extends Thread {
		private ServerSocket ss;
		public FrameSliderUpdater() {
			try {
				ss = new ServerSocket(FRAME_SLIDER_PORT);
			} catch (IOException e) {
				System.err.println(e);
			}
		}
		public void run() {
			while (true) {
				try {
					Socket s = ss.accept();
					BufferedReader fss = new BufferedReader(new InputStreamReader(s.getInputStream()));
					String str = fss.readLine();
					s.close();
					StringTokenizer t = new StringTokenizer(str, " \t\n\r");
					String fps = new String(t.nextToken());
					int value = 0;
					try {
						value = (new Integer(fps)).intValue();
					} catch (NumberFormatException nE) {
						System.err.println(nE);
					}
					frameSlider.setValue(value);
					//System.out.println("Get FPS = " + value);
				} catch (IOException e) {
					System.err.println(e);
				}
			}
		}
	}

	class LoadControlChange implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();


			if (!source.getValueIsAdjusting()) {
				int index = 0;
				for (int i=0; i < encListModel.getSize(); i++) {
					if (loadControlSlider[i] == source) {
						index = i;
					}
				}
				String hostName = encListModel.getElementAt(index).toString();
				if (hostName != "") {
					int loadControlValue = (int)((JSlider)e.getSource()).getValue();
/**/
					try {
						Socket loadControlSocket = new Socket(hostName, LOAD_CONTROL_PORT);
						PrintStream loadControlOutputStream = new PrintStream(loadControlSocket.getOutputStream());
						loadControlOutputStream.print("load "+loadControlValue);
						loadControlOutputStream.flush();
						loadControlSocket.close();
					} catch (UnknownHostException sE) {
						System.err.println(sE);
					} catch (IOException sE) {
						System.err.println(sE);
					}
/**/
					System.out.println("Load Control Value of " + hostName + " = " + loadControlValue);
				}
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

	public static void main(String[] args) {
		JFrame frame = new JFrame("ControlCenter");
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});

		frame.getContentPane().add(new JLabel("<html><h1 align=center>SONET: Active Subnet Diffusion Transcoder<br>Channel Control Center</h1>", JLabel.CENTER), BorderLayout.NORTH);
		frame.getContentPane().add(new ControlCenter(), BorderLayout.CENTER);

		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new GridLayout(2,1));
		bottomPanel.setPreferredSize(new Dimension(windowWidth, bottomWindowHeight));

		StatusPanel sp = new StatusPanel();
		bottomPanel.add((JPanel)sp);

		JPanel medianetPanel = new JPanel();
		medianetPanel.setLayout(new BoxLayout(medianetPanel, BoxLayout.X_AXIS));
//		JLabel authorLabel = new JLabel(new ImageIcon("author.gif"));
//		medianetPanel.add(authorLabel);
		JButton aboutButton = new JButton("About");
//		About aboutAction = new About();
		aboutButton.addActionListener(new About());
		medianetPanel.add(aboutButton);
		JButton medianetButton = new JButton(new ImageIcon("medianet.gif"));
		MedianetWeb medianetAction = new MedianetWeb();
		medianetButton.addActionListener(medianetAction);
		medianetPanel.add(Box.createHorizontalGlue());
		medianetPanel.add(medianetButton);
		bottomPanel.add(medianetPanel);

		frame.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
		frame.setSize(windowWidth, windowHeight);
		frame.setVisible(true);
		sp.Update();
	}
}
