#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <errno.h>


#define BUFFER_SIZE	1024
#define CMD_PORT	9000

char *progName = "DIXEE";
char *pidFileNamePrefix = "Pid";
char *logFileName = "ctrl.log";
char *bitRateFileName = "xlun/rate.par";

struct sockaddr_in svr, cli;
int s;
int s2;
int icli;
int fromlen;
int isVerbose = 0;
int isDaemon = 0;

main(int argc, char *argv[])
{
	char command[BUFFER_SIZE];
	char arg1[BUFFER_SIZE];
	char arg2[BUFFER_SIZE];


	ProcessOptions(argc, argv);

	//signal(SIGPIPE, SIG_IGN);

	s = socket(AF_INET, SOCK_STREAM, 0);

	bzero(&svr, sizeof(svr));
	svr.sin_family = AF_INET;
	svr.sin_addr.s_addr = htonl(INADDR_ANY);
	svr.sin_port = htons(CMD_PORT);

	bind(s, (struct sockaddr *) &svr, sizeof(svr));
	listen(s, 4096);

	while (1) {
		if (!isDaemon) {
			printf("[%s]: Ready.\n", progName);
		}
		s2 = accept(s, (struct sockaddr *) &cli, &icli);

		bzero(command, BUFFER_SIZE);
		recvfrom(s2, command, BUFFER_SIZE,
			 MSG_WAITALL | MSG_NOSIGNAL, NULL, &fromlen);

		if (strstr(command, "download") != NULL) {
			close(s2);
			if (!isDaemon) {
				printf("[Cmd] %s --- received\n", command);
			}
			sscanf(command, "%s %s", arg1, arg2);
			Download(arg2);
		} else if (strstr(command, "start") != NULL) {
			close(s2);
			if (!isDaemon) {
				printf("[Cmd] %s --- received\n", command);
			}
			sscanf(command, "%s %s", arg1, arg2);
			Run(arg2);
		} else if (strstr(command, "stop") != NULL) {
			close(s2);
			if (!isDaemon) {
				printf("[Cmd] %s --- received\n", command);
			}
			sscanf(command, "%s %s", arg1, arg2);
			Kill(arg2);
		} else if (strstr(command, "bitrate") != NULL) {
			close(s2);
			if (!isDaemon) {
				printf("[Cmd] %s --- received\n", command);
			}
			sscanf(command, "%s %s", arg1, arg2);
			MakeBitRate(atoi(arg2));
		} else {
			if (!isDaemon) {
				printf("%s: Unknown command %s", progName,
				       command);
			}
			close(s2);
		}
	}
}


Download(char *fileName)
{
	FILE *f;
	int i;
	char buf[BUFFER_SIZE];

	bzero(buf, BUFFER_SIZE);
	if ((f = fopen(fileName, "w")) == NULL) {
		if (!isDaemon) {
			printf("%s: Cannot create %s\n", progName,
			       fileName);
		}
		return;
	}

	s2 = accept(s, (struct sockaddr *) &cli, &icli);

	if (!isDaemon) {
		printf("[Load] %s: ", fileName);
		fflush(stdout);
	}

	i = recvfrom(s2, buf, BUFFER_SIZE, MSG_WAITALL | MSG_NOSIGNAL,
		     NULL, &fromlen);
	while (i > 0) {
		if (!isDaemon) {
			printf(".");
			fflush(stdout);
		}
		fwrite(buf, i, 1, f);
		fflush(f);
		i = recvfrom(s2, buf, BUFFER_SIZE,
			     MSG_WAITALL | MSG_NOSIGNAL, NULL, &fromlen);
	}
	if (!isDaemon) {
		printf(" done\n");
		fflush(stdout);
	}
	close(s2);
	fclose(f);
}

Run(char *prog)
{
	char dir[BUFFER_SIZE];
	char pidFileName[BUFFER_SIZE];
	static char runName[BUFFER_SIZE];
	char buf[BUFFER_SIZE];
	int pid;
	FILE *f;

	bzero(dir, BUFFER_SIZE);
	bzero(runName, BUFFER_SIZE);
	bzero(pidFileName, BUFFER_SIZE);
	strcat(pidFileName, pidFileNamePrefix);
	if (strcmp(prog, "svr") == 0) {
		strcat(pidFileName, "Svr");
		strcat(dir, "./svr");
		strcat(runName, "./svr");
	} else if (strcmp(prog, "dec") == 0) {
		strcat(pidFileName, "Dec");
		strcat(dir, "./test");
		strcat(runName, "../xdec/mpeg2decode");
	} else if (strcmp(prog, "enc") == 0) {
		strcat(pidFileName, "Enc");
		strcat(dir, "./xlun");
		strcat(runName, "./xlun");
	} else if (strcmp(prog, "mux") == 0) {
		strcat(pidFileName, "Mux");
		strcat(dir, "./xmux");
		strcat(runName, "./xmux");
	} else {
		if (!isDaemon) {
			fprintf(stderr, "%s: Unknown application.\n",
				progName);
		}
		return -1;
	}

	 /**/ pid = fork();
	if (pid == -1) {
		if (!isDaemon) {
			fprintf(stderr, "%s: Error in fork.\n", progName);
		}
	} else if (pid == 0) {
		int ret;
		int lf;

		chdir(dir);
		if ((!isDaemon) && isVerbose) {
			close(1);
			if ((lf =
			     open(logFileName,
				  O_CREAT | O_TRUNC | O_WRONLY,
				  0666)) == -1) {
				if (!isDaemon) {
					fprintf(stderr,
						"%s: Error in open log file(%s).\n",
						progName, logFileName);
				}
			} else {
				close(2);
				dup(1);
			}
		}
		if (!isDaemon) {
			printf("[Run] %s (%s)\n", pidFileName, runName);
		}
		if (strcmp(prog, "dec") != 0) {
			ret = execlp(runName, runName);
		} else {
			char arg1[] = "-b";
			char arg2[] = "test.m2v";
			char arg3[] = "-r";
			char arg4[] = "-f";
			char arg5[] = "-Q";
			char arg6[] = "-o0";
			char arg7[] = "mid%d";
			char arg8[] = "1";
			ret =
			    execlp(runName, runName, arg1, arg2, arg3,
				   arg4, arg5, arg6, arg7, arg8);
			if (!isDaemon) {
				printf
				    ("[Run] %s %s %s %s %s %s %s %s %s\n",
				     runName, arg1, arg2, arg3, arg4, arg5,
				     arg6, arg7, arg8);
			}
		}
		if (ret == -1) {
			if (!isDaemon) {
				fprintf(stderr,
					"%s: Error in execlp(%s) errno(%d).\n",
					progName, runName, errno);
				perror("Error Explain:");
			}
		}
		exit(0);
	} else {
		if ((f = fopen(pidFileName, "w")) == NULL) {
			if (!isDaemon) {
				fprintf(stderr,
					"%s: Error in writing Pid file.\n",
					progName);
			}
			return -1;
		}
		bzero(buf, BUFFER_SIZE);
		sprintf(buf, "%d\n", pid);
		fwrite(buf, strlen(buf) + 1, 1, f);
		fclose(f);
	}
}

Kill(char *prog)
{
	FILE *f;
	int pid;
	int status;
	char buf[BUFFER_SIZE];
	char dir[BUFFER_SIZE];
	char runName[BUFFER_SIZE];
	char pidFileName[BUFFER_SIZE];

	bzero(pidFileName, BUFFER_SIZE);
	bzero(dir, BUFFER_SIZE);
	bzero(runName, BUFFER_SIZE);
	strcat(pidFileName, pidFileNamePrefix);
	if (strcmp(prog, "svr") == 0) {
		strcat(pidFileName, "Svr");
		strcat(dir, "./svr");
	} else if (strcmp(prog, "dec") == 0) {
		strcat(pidFileName, "Dec");
		strcat(dir, "./test");
		strcat(runName, "./clean");
	} else if (strcmp(prog, "enc") == 0) {
		strcat(pidFileName, "Enc");
		strcat(dir, "./xlun");
		strcat(runName, "./clean");
	} else if (strcmp(prog, "mux") == 0) {
		strcat(pidFileName, "Mux");
		strcat(dir, "./xmux");
		strcat(runName, "./clean");
	} else {
		if (!isDaemon) {
			fprintf(stderr, "%s: Unknown application.\n",
				progName);
		}
		return -1;
	}

	if (!isDaemon) {
		printf("[Kill] %s\n", pidFileName);
	}

	/**/ bzero(buf, BUFFER_SIZE);
	if ((f = fopen(pidFileName, "r")) == NULL) {
		if (!isDaemon) {
			fprintf(stderr, "%s: Pid file can't open\n",
				progName);
		}
		return -1;
	}
	fscanf(f, "%d", &pid);
	fclose(f);

	//kill(pid, SIGHUP);
	kill(pid, SIGTERM);
	//kill(pid, SIGKILL);
	waitpid(pid, NULL, 0);
	unlink(pidFileName);
	if (strcmp(runName, "./clean") == 0) {
		pid = fork();
		if (pid == -1) {
			if (!isDaemon) {
				fprintf(stderr, "%s: Error in fork.\n",
					progName);
			}
		} else if (pid == 0) {
			int ret;

			chdir(dir);
			if (!isDaemon) {
				printf("[Kill] running (%s)\n", runName);
			}
			ret = execlp(runName, runName);
			exit(0);
		} else {
			wait(NULL);
		}
	}
}

MakeBitRate(int bitRate)
{
	FILE *fp;

	if ((fp = fopen(bitRateFileName, "w")) == NULL) {
		if (!isDaemon) {
			printf("[BitRate] Error open file %s\n", bitRateFileName);
		}
		return 0;
	}
	fprintf(fp, "%lf\n", (float)(1000.0 * bitRate));
	fclose(fp);
}

ProcessOptions(int argc, char *argv[])
{
	int i;

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (toupper(argv[i][1])) {
			case 'V':
				if (isDaemon) {
					fprintf(stderr,
						"Error! Couldn't set both -d and -v.\n");
					exit(1);
				} else {
					isVerbose = 1;
				}
				break;
			case 'D':
				if (isVerbose) {
					fprintf(stderr,
						"Error! Couldn't set both -d and -v.\n");
					exit(1);
				} else {
					isDaemon = 1;
				}
				break;
			default:
				fprintf(stderr, "Unknown argument(%s).\n",
					argv[i]);
				break;
			}
		} else {
			fprintf(stderr, "Unknown argument(%s).\n",
				argv[i]);
		}
	}
}
