
/* -------------------------------------------------------------------- */
/*                                                                      */
/*  Program Name  : X Encoder Luncher                                   */
/*  Author        : Seung-su Yang                                       */
/*  Version       : 1.0                                                 */
/*  Date          : Nov. 25, 2000                                       */
/*                                                                      */
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <pthread.h>
#include <netdb.h>
#include <time.h>
#include <wait.h>
#include <signal.h>

#define BACKLOG	4096
#define BUFFERSIZE 1024
#define ENCODER_QUEUE_LENGTH 100

// declare functions
void GetConfigInfo(void);
void OpenDataServer();
void OpenCmdServer();
void RecvCmd();
void CallEncoder(int);
void SendAck();
void SendM2V(int);
void Ending();
void GetMandN();
void *RunEncoder(void *);

// defind global variables

char progName[] = "EncoderLuncher";
char confName[] = "PXNet.conf";
char subPidFile[] = "PidXlunSub";
int xEncLuncherCmdPort;
int xEncLuncherDataPort;
char xMuxName[BUFFERSIZE];
char xDecName[BUFFERSIZE];
int xMuxPort;
int xDecCtrlPort;
char *xHomeDir;

int cmdListenSocket;

int EncQueue[ENCODER_QUEUE_LENGTH];
int qS = 0, qE = 0;

int M, N;

int cmdSocket;
int dataSocket;

void quit(int sig)
{
	FILE *f;
	int subPid;

	if ((f = fopen(subPidFile, "r")) != NULL) {
		fscanf(f, "%d", &subPid);
		fclose(f);
		kill(subPid, SIGTERM);
		//kill(subPid, SIGKILL);
	}
	unlink(subPidFile);
	close(cmdSocket);
	close(cmdListenSocket);
	exit(0);
}

void quit2(int sig)
{
	printf("receiving signal (%d)\n", sig); fflush(stdout);
	close(dataSocket);
	close(cmdSocket);
	close(cmdListenSocket);
	exit(0);
}


main()
{
	pthread_t tId;


	signal(SIGTERM, quit);

	GetConfigInfo();
	GetMandN();
	OpenDataServer();
	OpenCmdServer();
	if (pthread_create(&tId, NULL, RunEncoder, NULL)) {
		fprintf(stderr, "%s : Error in creating thread\n", progName);
	}
	pthread_detach(tId);
	while (1) {
		RecvCmd();
	}
	Ending();

}

void GetConfigInfo()
{
	FILE *fp;
	char buffer[BUFFERSIZE];
	char xVideoServerName[BUFFERSIZE];
	char xVideoPlayerName[BUFFERSIZE];
	int xVideoServerPort;
	int xVideoPlayerPort;

	xHomeDir = getenv("XHOMEDIR");
	if (xHomeDir == NULL) {
		fprintf(stderr, "%s : XHOMEDIR is not assigned.\n", progName);
		xHomeDir = "..";
	}

	bzero(buffer, BUFFERSIZE);
	sprintf(buffer, "%s/%s", xHomeDir, confName);
	if ((fp = fopen(buffer, "r")) < 0) {
		fprintf(stderr, "%s : Error in open a network configuration file(%s).\n", progName, buffer);
		exit(1);
	} else {
		bzero(buffer, BUFFERSIZE);
		bzero(xMuxName, BUFFERSIZE);

		// X Video Server
		fscanf(fp, "%s%d", xVideoServerName, &xVideoServerPort);
#ifdef DEBUG
		printf("%s : X Video Server(%s), Port(%d)\n", progName, xVideoServerName, xVideoServerPort);
#endif

		// X Video Player
		fscanf(fp, "%s%d", xVideoPlayerName, &xVideoPlayerPort);
#ifdef DEBUG
		printf("%s : X Video Player(%s), Port(%d)\n", progName, xVideoPlayerName, xVideoPlayerPort);
#endif

		// X encoder luncher
		fscanf(fp, "%s%d%d", buffer, &xEncLuncherCmdPort, &xEncLuncherDataPort);
#ifdef DEBUG
		printf("%s : Command Port(%d), Data Port(%d)\n", progName, xEncLuncherCmdPort, xEncLuncherDataPort);
#endif

		// X decoder
		fscanf(fp, "%s%d", xDecName, &xDecCtrlPort);
#ifdef DEBUG
		printf("%s : xDecName(%s), ctrlPort(%d)\n", progName, xDecName, xDecCtrlPort);
#endif

		// X multiplexer
		fscanf(fp, "%s%d", xMuxName, &xMuxPort);
#ifdef DEBUG
		printf("%s : xMuxName(%s), Port(%d)\n", progName, xMuxName, xMuxPort);
#endif
	}
	fclose(fp);
}

void OpenDataServer()
{
	// for data socket
	int dataListenSocket;
	struct sockaddr_in dataListenAddr, dataClientAddr;
	struct sockaddr dataFromAddr;
	int dataClientAddrLength, dataFromAddrLength;
	char buffer[BUFFERSIZE];

	int fd;
	int pid;
	int ret;
	
	pid = fork();
	if (pid < 0) {
		fprintf(stderr, "%s : Error on fork Data server.\n", progName);
	} else if (pid == 0) {
		signal(SIGTERM, quit2);
		// for data port
		dataListenSocket = socket(AF_INET, SOCK_STREAM, 0);
		bzero(&dataListenAddr, sizeof(dataListenAddr));
		dataListenAddr.sin_family = AF_INET;
		dataListenAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		dataListenAddr.sin_port = htons(xEncLuncherDataPort);
		bind(dataListenSocket, (struct sockaddr *)&dataListenAddr, sizeof(dataListenAddr));
		listen(dataListenSocket, BACKLOG);

		while(1) {
			// for file name
			dataSocket = accept(dataListenSocket, (struct sockaddr *)&dataClientAddr, &dataClientAddrLength);
			bzero(buffer, BUFFERSIZE);
			recvfrom(dataSocket, buffer, BUFFERSIZE, 0, &dataFromAddr, &dataFromAddrLength);
			
			// create file
			if ((fd = creat(buffer, S_IRWXU | S_IRWXG)) < 0) {
				fprintf(stderr, "%s : Error in create file %s\n", buffer);
				exit(1);
			}
#ifdef DEBUG
#ifdef VERBOSE
			printf("%s : Receiving %s file\n", progName, buffer);
#endif
#endif
			bzero(buffer, BUFFERSIZE);
			while (ret = recvfrom(dataSocket, buffer, BUFFERSIZE, 0, &dataFromAddr, &dataFromAddrLength)) {
				write(fd, buffer, ret);
			}
			close(fd);
			close(dataSocket);
		}
		unlink(subPidFile);
	} else {
		FILE *f;
		if ((f = fopen(subPidFile, "w")) == NULL) {
			fprintf(stderr, "%s: %s open error for writing.\n", progName, subPidFile);
		} else {
			fprintf(f, "%d", pid);
		}
		fclose(f);
	}
	close(dataListenSocket);
}

void OpenCmdServer()
{
	// for command socket
	struct sockaddr_in cmdListenAddr;

	// for command port
	cmdListenSocket = socket(AF_INET, SOCK_STREAM, 0);
	bzero(&cmdListenAddr, sizeof(cmdListenAddr));
	cmdListenAddr.sin_family = AF_INET;
	cmdListenAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	cmdListenAddr.sin_port = htons(xEncLuncherCmdPort);
	bind(cmdListenSocket, (struct sockaddr *)&cmdListenAddr, sizeof(cmdListenAddr));
	listen(cmdListenSocket, BACKLOG);

	printf("%s : ready\n", progName);

}

void RecvCmd()
{

	struct sockaddr_in cmdClientAddr;
	int cmdClientAddrLength;
	struct sockaddr cmdFromAddr;
	int cmdFromAddrLength;

	char buffer[BUFFERSIZE];
	int i;
	int fileNo;
	int fd;
	int ret;
	int gopNo;
	time_t t;

	// receive GOP number and file number
	cmdSocket = accept(cmdListenSocket, (struct sockaddr *)&cmdClientAddr, &cmdClientAddrLength);
	bzero(buffer, BUFFERSIZE);
	recvfrom(cmdSocket, buffer, BUFFERSIZE, 0, &cmdFromAddr, &cmdFromAddrLength);
	sscanf(buffer, "%d", &gopNo);
	close(cmdSocket);

#ifdef DEBUG
	time(&t);
	printf("%s : Receiving gopNo(%d) %s\n", progName, gopNo, ctime(&t));
	fflush(stdout);
#endif
	EncQueue[qE] = gopNo;
	qE = (qE + 1) % ENCODER_QUEUE_LENGTH;

}

void CallEncoder(int gopNo)
{
	int pid;
	char buffer[BUFFERSIZE];
	char runProgram[BUFFERSIZE];
	char arg1[BUFFERSIZE];
	char arg2[BUFFERSIZE];
	char arg3[BUFFERSIZE];
	char arg4[BUFFERSIZE];
	int i, j;
	time_t t;
	int iret;

	pid = fork();
	if (pid == -1) {
		fprintf(stderr, "%s : Error in fork\n", progName);
		SendM2V(gopNo);
	} else if (pid == 0) {

		bzero(buffer, BUFFERSIZE);
		// make invoke encoder command
		sprintf(runProgram, "%s/xenc/mpeg2encode", xHomeDir);
		sprintf(arg1, "test.par");
		sprintf(arg2, "m%03d.m2v", gopNo);
		sprintf(arg3, "1");
		sprintf(arg4, "%d", gopNo);

#ifdef DEBUG
		time(&t);
		runProgram[strlen(runProgram)+1] = '\0';
		arg1[strlen(arg1)+1] = '\0';
		arg2[strlen(arg2)+1] = '\0';
		arg3[strlen(arg3)+1] = '\0';
		arg4[strlen(arg4)+1] = '\0';
		printf("%s : Invoking Encoder. %s %s %s %s %s (%s)\n", progName, runProgram, arg1, arg2, arg3, arg4, ctime(&t));
		fflush(stdout);
#endif
		// invoke encoder
		//system(buffer);
		iret = execlp(runProgram, runProgram, arg1, arg2, arg3, arg4);
		printf("Runned %d\n", iret);
		exit(0);

	} else {
		printf("Waiting %d\n", pid);
		fflush(stdout);
		wait(NULL);
		printf("Process End %d\n", pid);
		fflush(stdout);
		// send Ack to Decoder
		SendAck(gopNo);
		// send encoded data
		SendM2V(gopNo);

		// delete data
		i = (gopNo) ? (gopNo+1)*N - M: N - M;
		for (j = (gopNo) ? (gopNo)*N - M + 1 : 0; j <= i; j++) {
			bzero(buffer, BUFFERSIZE);
			sprintf(buffer, "mid%d.Y", j);
			unlink(buffer);
			bzero(buffer, BUFFERSIZE);
			sprintf(buffer, "mid%d.U", j);
			unlink(buffer);
			bzero(buffer, BUFFERSIZE);
			sprintf(buffer, "mid%d.V", j);
			unlink(buffer);
		}
		bzero(buffer, BUFFERSIZE);
		sprintf(buffer, "pskfile%02d", gopNo);
		unlink(buffer);
		//exit(0);
	}
}

void SendAck(int gopNo)
{
	int ackSocket;
	struct sockaddr_in ackAddr;
	struct hostent *hostEnt;
	char buffer[BUFFERSIZE];

	hostEnt = gethostbyname(xDecName);
	ackSocket = socket(AF_INET, SOCK_STREAM, 0);
	bzero(&ackAddr, sizeof(ackAddr));
	ackAddr.sin_family = AF_INET;
	memcpy(&(ackAddr.sin_addr), hostEnt->h_addr, hostEnt->h_length);
	ackAddr.sin_port = htons(xDecCtrlPort+1);
	if (connect(ackSocket, (struct sockaddr *)&ackAddr, sizeof(ackAddr)) == -1) {
		fprintf(stderr, "%s : Error in connect xDec Ack (%s:%d)\n", progName, xDecName, xDecCtrlPort);
		exit(1);
	}
	bzero(buffer, BUFFERSIZE);
	sprintf(buffer, "%d", gopNo);
	write(ackSocket, buffer, BUFFERSIZE);
	close(ackSocket);
}

void SendM2V(int gopNo)
{

	// for command socket
	int m2vSocket;
	struct sockaddr_in m2vAddr;
	
	// common use
	struct hostent *hostEnt;

	int fd;
	int ret;
	char buffer[BUFFERSIZE];
	time_t t;

	hostEnt = gethostbyname(xMuxName);

	// send GOP No.
	m2vSocket = socket(AF_INET, SOCK_STREAM, 0);
	bzero(&m2vAddr, sizeof(m2vAddr));
	m2vAddr.sin_family = AF_INET;
	memcpy(&(m2vAddr.sin_addr), hostEnt->h_addr, hostEnt->h_length);
	m2vAddr.sin_port = htons(xMuxPort);
	if (connect(m2vSocket, (struct sockaddr *)&m2vAddr, sizeof(m2vAddr)) == -1) {
		fprintf(stderr, "%s : Error in connect xMux command (%s:%d)\n", progName, xMuxName, xMuxPort);
		exit(1);
	}
	bzero(buffer, BUFFERSIZE);
	sprintf(buffer, "%d", gopNo);
	write(m2vSocket, buffer, BUFFERSIZE);
	
	bzero(buffer, BUFFERSIZE);
	sprintf(buffer, "m%03d.m2v", gopNo);
	// send encoded data
	if ((fd = open(buffer, O_RDONLY)) < 0) {
		fprintf(stderr, "%s : Error in open a file %s \n", progName, buffer);
		close(m2vSocket);
	} else {
#ifdef DEBUG
		time(&t);
		printf("%s : %s file opened.%s\n", progName, buffer, ctime(&t));
		fflush(stdout);
#endif
		bzero(buffer, BUFFERSIZE);
		while (ret = read(fd, buffer, BUFFERSIZE)) {
			ret = write(m2vSocket, buffer, ret);
		}
		close(m2vSocket);
		close(fd);
	}
	bzero(buffer, BUFFERSIZE);
	sprintf(buffer, "m%03d.m2v", gopNo);
	unlink(buffer);
}

void Ending()
{
	close(cmdListenSocket);
}

void GetMandN()
{
	FILE *fp;
	char buffer[BUFFERSIZE];

	if ((fp = fopen("test.par", "r")) == NULL) {
		printf("%s : Error Coundn't open test.par file\n", progName);
		exit(1);
	}
	bzero(buffer, BUFFERSIZE);
	fgets(buffer, BUFFERSIZE, fp);
	bzero(buffer, BUFFERSIZE);
	fgets(buffer, BUFFERSIZE, fp);
	sscanf(buffer, "%d", &N);
	bzero(buffer, BUFFERSIZE);
	fgets(buffer, BUFFERSIZE, fp);
	sscanf(buffer, "%d", &M);
	fclose(fp);
	printf("%s : M (%d), N(%d)\n", progName, M, N);
}

void *RunEncoder(void *p)
{
	char buffer[BUFFERSIZE];
	FILE *fp;
	int i;
	int gopNo;

	while (1) {
		if (qS != qE) {
			gopNo = EncQueue[qS];
			// wait until all data is available
			bzero(buffer, BUFFERSIZE);
			// waiting file
			i = (gopNo+1) * N - M;
			sprintf(buffer, "mid%d.V", i);
			//printf("%s : Testing %s file \n", progName, buffer);
			while ((fp = fopen(buffer, "r")) == NULL) {
#ifdef DEBUG
			//printf("%s : waiting\n", progName);
#endif
				sleep(1);
			}
			fclose(fp);
			CallEncoder(gopNo);
			qS = (qS + 1) % ENCODER_QUEUE_LENGTH;
		}
	}

}
