
/* -------------------------------------------------------------------- */
/*                                                                      */
/*  Program Name  : X Multiplexer                                       */
/*  Author        : Seung-su Yang                                       */
/*  Version       : 1.0                                                 */
/*  Date          : Nov. 25, 2000                                       */
/*                                                                      */
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pthread.h>
#include <time.h>
#include <sys/timeb.h>
#include <signal.h>


#define FALSE	0
#define TRUE	1
#define BUFFERSIZE	1024
#define BACKLOG	4096
#define FPSTIMESIZE 20
#define GUARDED_GOP_NO 10

void GetConfigInfo();
void *GetData(void *);
void *SendData(void *);
void *InitializeCurrent(void *arg);

char progName[] = "xMux";
char confName[] = "PXNet.conf";
const char controlCenterName[] = "131.123.36.46";
//char outFileName[] = "m.m2v";
int xVideoPlayerPort;
int xEncLuncherCmdPort;
int xEncLuncherDataPort;
char xMuxName[BUFFERSIZE];
int xMuxPort;

int gop[BUFFERSIZE];
int current = 0;
int semaphore = 0;
struct timeb t1, t2;
int totalTime = 0;
int framePerGop = 1;
struct timeb fpsTime[FPSTIMESIZE];
int fpsTimeIndex = 0;

int cmdSocket;
int playerSocket;

void quit(int sig)
{
    close(cmdSocket);
    close(playerSocket);
    exit(0);
}

main()
{
    int cmdListenSocket;
    struct sockaddr_in cmdListenAddr, cmdClientAddr;
    int cmdClientLength;
    int i;
    pthread_t tId;

    signal(SIGTERM, quit);
    for (i = 0; i < BUFFERSIZE; i++) {
	gop[i] = FALSE;
    }

    GetConfigInfo();
    GetGopSize();

    // Mux Reset
    if (pthread_create(&tId, NULL, InitializeCurrent, NULL)) {
	fprintf(stderr, "%s : Error in create thread for Reset\n",
		progName);
    }
    pthread_detach(tId);
    tId++;

    // for command
    cmdListenSocket = socket(AF_INET, SOCK_STREAM, 0);
    bzero(&cmdListenAddr, sizeof(cmdListenAddr));
    cmdListenAddr.sin_family = AF_INET;
    cmdListenAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    cmdListenAddr.sin_port = htons(xMuxPort);
    bind(cmdListenSocket, (struct sockaddr *) &cmdListenAddr,
	 sizeof(cmdListenAddr));
    listen(cmdListenSocket, BACKLOG);

    // Send Data
     /**/ if (pthread_create(&tId, NULL, SendData, NULL)) {
	fprintf(stderr, "%s : Error in create thread for SendData\n",
		progName);
    }
    pthread_detach(tId);
    tId++;
     /**/ while (1) {
#ifdef DEBUG
	printf("%s : waiting command\n", progName);
#endif
	cmdSocket =
	    accept(cmdListenSocket, (struct sockaddr *) &cmdClientAddr,
		   &cmdClientLength);
	if (pthread_create(&tId, NULL, GetData, (void *) (&cmdSocket))) {
	    fprintf(stderr, "%s : Error in fork\n", progName);
	}
	pthread_detach(tId);
	tId++;
    }
    close(cmdListenSocket);
}

void GetConfigInfo()
{
    FILE *fp;
    char *xHomeDir;
    char buffer[BUFFERSIZE];
    char xVideoServerName[BUFFERSIZE];
    char xVideoPlayerName[BUFFERSIZE];
    char xDecName[BUFFERSIZE];
    int xVideoServerPort;
    int xDecCtrlPort;

    xHomeDir = getenv("XHOMEDIR");
    if (xHomeDir == NULL) {
	printf("%s : XHOMEDIR is not assigned.\n", progName);
	xHomeDir = "..";
    }

    bzero(buffer, BUFFERSIZE);
    sprintf(buffer, "%s/%s", xHomeDir, confName);
    if ((fp = fopen(buffer, "r")) < 0) {
	printf("%s : Error in open a network configuration file(%s).\n",
	       progName, buffer);
	exit(1);
    } else {
	bzero(buffer, BUFFERSIZE);
	bzero(xMuxName, BUFFERSIZE);

	// X Video Server
	fscanf(fp, "%s%d", xVideoServerName, &xVideoServerPort);
#ifdef DEBUG
	printf("%s : X Video Server(%s), Port(%d)\n", progName,
	       xVideoServerName, xVideoServerPort);
#endif

	// X Video Player
	fscanf(fp, "%s%d", xVideoPlayerName, &xVideoPlayerPort);
#ifdef DEBUG
	printf("%s : X Video Player(%s), Port(%d)\n", progName,
	       xVideoPlayerName, xVideoPlayerPort);
#endif

	// X encoder luncher
	fscanf(fp, "%s%d%d", buffer, &xEncLuncherCmdPort,
	       &xEncLuncherDataPort);
#ifdef DEBUG
	printf("%s : Command Port(%d), Data Port(%d)\n", progName,
	       xEncLuncherCmdPort, xEncLuncherDataPort);
#endif

	// X decoder
	fscanf(fp, "%s%d", xDecName, &xDecCtrlPort);
#ifdef DEBUG
	printf("%s : xDecName(%s), Port(%d)\n", progName, xDecName,
	       xDecCtrlPort);
#endif

	// X multiplexer
	fscanf(fp, "%s%d", xMuxName, &xMuxPort);
#ifdef DEBUG
	printf("%s : xMuxName(%s), Port(%d)\n", progName, xMuxName,
	       xMuxPort);
#endif
    }
    fclose(fp);
}

GetGopSize()
{
    FILE *fp;

    if ((fp = fopen("gopsize", "r")) < 0) {
	printf("%s : Error in open a GOP size file(%s).\n", progName,
	       "gopsize");
    } else {
	fscanf(fp, "%d", &framePerGop);
    }
}
void *GetData(void *arg)
{
    int s;
    int fd, fd2;
    char buffer[BUFFERSIZE];
    char fileName[BUFFERSIZE];
    int gopNo;
    int ret;
    int fps;

    struct sockaddr_in cmdListenAddr, cmdClientAddr;
    struct sockaddr cmdFromAddr;
    int cmdFromLength;
    int cmdClientLength;

    s = *((int *) arg);
    printf("%s : Running %d \n", progName, s);

    if (current == 0) {
	ftime(&t1);
	t2 = t1;
	totalTime = 0;
    }
    // receive gop no
    bzero(buffer, BUFFERSIZE);
    recvfrom(s, buffer, BUFFERSIZE, 0, &cmdFromAddr, &cmdFromLength);
    sscanf(buffer, "%d", &gopNo);
    ftime(&t1);
    memcpy(&(fpsTime[fpsTimeIndex % FPSTIMESIZE]), &t1,
	   sizeof(struct timeb));
    totalTime = totalTime + (t1.time - t2.time);
    printf
	("%s : Time/GOP(%d), AVR/GOP (%d), Time/Frame(%f), AVR/Frame(%f), Elapsed Time(%d)\n",
	 progName, t1.time - t2.time, (int) (totalTime / (current + 1)),
	 ((float) (t1.time - t2.time) / framePerGop),
	 ((float) totalTime / (framePerGop * (current + 1))), totalTime);

    if (fpsTimeIndex < FPSTIMESIZE) {
	if ((int) (fpsTime[fpsTimeIndex].time - fpsTime[0].time) <= 0) {
	    if ((int) (fpsTime[fpsTimeIndex].millitm - fpsTime[0].millitm)
		<= 0) {
		fps = framePerGop * (fpsTimeIndex + 1);
	    } else {
		fps =
		    framePerGop * (fpsTimeIndex +
				   1) /
		    (float) ((fpsTime[fpsTimeIndex].millitm -
			      fpsTime[0].millitm) * 0.001);
		printf
		    ("%s : t1.time %d, t2.time %d, t1.mtm %d, t2.mtm %d\n",
		     progName, fpsTime[fpsTimeIndex].time, fpsTime[0].time,
		     fpsTime[fpsTimeIndex].millitm, fpsTime[0].millitm);
	    }
	} else {
	    fps =
		framePerGop * (fpsTimeIndex +
			       1) / (float) (fpsTime[fpsTimeIndex].time -
					     fpsTime[0].time);
	}
    } else {
	if ((int)
	    (fpsTime[fpsTimeIndex % FPSTIMESIZE].time -
	     fpsTime[(fpsTimeIndex + 1) % FPSTIMESIZE].time) <= 0) {
	    if ((int)
		(fpsTime[fpsTimeIndex % FPSTIMESIZE].millitm -
		 fpsTime[(fpsTimeIndex + 1) % FPSTIMESIZE].millitm) <= 0) {
		fps = framePerGop * FPSTIMESIZE;
	    } else {
		fps =
		    framePerGop * FPSTIMESIZE /
		    (float) ((fpsTime[fpsTimeIndex % FPSTIMESIZE].millitm -
			      fpsTime[(fpsTimeIndex +
				       1) % FPSTIMESIZE].millitm) * 0.001);
		printf
		    ("%s : t1.time %d, t2.time %d, t1.mtm %d, t2.mtm %d\n",
		     progName, fpsTime[fpsTimeIndex % FPSTIMESIZE].time,
		     fpsTime[(fpsTimeIndex + 1) % FPSTIMESIZE].time,
		     fpsTime[fpsTimeIndex % FPSTIMESIZE].millitm,
		     fpsTime[(fpsTimeIndex + 1) % FPSTIMESIZE].millitm);
	    }
	} else {
	    fps =
		framePerGop * FPSTIMESIZE /
		(float) (fpsTime[fpsTimeIndex % FPSTIMESIZE].time -
			 fpsTime[(fpsTimeIndex + 1) % FPSTIMESIZE].time);
	}
    }

    SendFps(fps);

    fpsTimeIndex++;
    t2 = t1;
    // make file name
    bzero(fileName, BUFFERSIZE);
    sprintf(fileName, "m%03d.m2v", gopNo);
    printf("Filename : %s\n", fileName);

    // create file
    fd = creat(fileName, S_IRWXU | S_IRWXG);
    bzero(buffer, BUFFERSIZE);
    while (ret =
	   recvfrom(s, buffer, BUFFERSIZE, 0, &cmdFromAddr,
		    &cmdFromLength)) {
	write(fd, buffer, ret);
	bzero(buffer, BUFFERSIZE);
    }
    gop[gopNo] = TRUE;
    close(s);
    close(fd);

    // GUARDED PROTECTION
    if (gopNo >= current + GUARDED_GOP_NO)
	current = current + 1;
    fflush(stdout);
}

void *SendData(void *arg)
{
    int fd;
    int ret;
    int playerListenSocket;
    struct sockaddr_in playerListenAddr, playerClientAddr;
    int playerClientLength;
    char fileName[BUFFERSIZE];
    char buffer[BUFFERSIZE];

    // for Player connection
    playerListenSocket = socket(AF_INET, SOCK_STREAM, 0);
    bzero(&playerListenAddr, sizeof(playerListenAddr));
    playerListenAddr.sin_family = AF_INET;
    playerListenAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    playerListenAddr.sin_port = htons(xVideoPlayerPort);
    bind(playerListenSocket, (struct sockaddr *) &playerListenAddr,
	 sizeof(playerListenAddr));
    listen(playerListenSocket, BACKLOG);

    playerSocket =
	accept(playerListenSocket,
	       (struct sockaddr *) &playerClientAddr, &playerClientLength);
    // sequantialize
    while (1) {
	printf("%s : current %d\n", progName, current);
	while (gop[current] == FALSE) {
	    ;			// wait
	}
	while (gop[current] == TRUE) {
	    bzero(fileName, BUFFERSIZE);
	    sprintf(fileName, "m%03d.m2v", current);
	    if ((fd = open(fileName, O_RDONLY)) < 0) {
		fprintf(stderr, "%s : Error in open %s file.\n", progName,
			fileName);
		exit(1);
	    }
	    while (ret = read(fd, buffer, BUFFERSIZE))
		write(playerSocket, buffer, ret);
	    close(fd);
	    unlink(fileName);
	    current++;
/*
			printf("%s : 2nd current %d\n", progName, current);
*/
	}
    }
    close(playerSocket);
}

void *InitializeCurrent(void *arg)
{
    int s;
    int sListenSocket;
    struct sockaddr_in sListenAddr, sClientAddr;
    struct sockaddr fromAddr;
    int sClientLength, fromLength;
    char buffer[BUFFERSIZE];
    int util;
    int i;

    sListenSocket = socket(AF_INET, SOCK_STREAM, 0);
    bzero(&sListenAddr, sizeof(sListenAddr));
    sListenAddr.sin_family = AF_INET;
    sListenAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    sListenAddr.sin_port = htons(xMuxPort + 1);
    bind(sListenSocket, (struct sockaddr *) &sListenAddr,
	 sizeof(sListenAddr));
    listen(sListenSocket, BACKLOG);

    while (1) {
	s = accept(sListenSocket, (struct sockaddr *) &sClientAddr,
		   &sClientLength);
	printf("%s : Resetting current\n", progName);
	bzero(buffer, BUFFERSIZE);
	recvfrom(s, buffer, BUFFERSIZE, 0, &fromAddr, &fromLength);
	close(s);
	sscanf(buffer, "%d", &current);
	for (i = 0; i < BUFFERSIZE; i++) {
	    gop[i] = FALSE;
	}
	current = 0;
	GetGopSize();
	printf("%s : Current Resetted \n", progName);
    }
    close(sListenSocket, BACKLOG);
}

SendFps(int fps)
{
    int controlCenterPort = 7001;
    struct sockaddr_in sAddr;
    struct hostent *hostEnt;
    char buffer[BUFFERSIZE];
    int s;
    time_t t;

    s = socket(AF_INET, SOCK_STREAM, 0);
    bzero(&sAddr, sizeof(sAddr));
    hostEnt = gethostbyname(controlCenterName);
    sAddr.sin_family = AF_INET;
    memcpy(&(sAddr.sin_addr), hostEnt->h_addr, hostEnt->h_length);
    sAddr.sin_port = htons(controlCenterPort);
    if (connect(s, (struct sockaddr *) &sAddr, sizeof(sAddr)) == -1) {
	fprintf(stderr, "%s : Error Sending FPS(%s:%d)\n", progName,
		controlCenterName, controlCenterPort);
    }
    bzero(buffer, BUFFERSIZE);
    sprintf(buffer, "%d ", fps);
    write(s, buffer, strlen(buffer) + 1);
    time(&t);
    printf("%s : FPS = %s %s %d\n", progName, buffer, ctime(&t),
	   (long int) t);
    close(s);
}
